import { fromObject, } from "@nativescript/core";
export class InternalEventManager {
    constructor() {
        this.notificationCenter = fromObject({});
        this.listenerCounter = 0;
        this.callbacks = new CallbackStore();
    }
    on(eventName, callback) {
        const callbackId = [eventName, this.listenerCounter];
        const internalCallback = (eventData) => callback(eventData.data);
        this.callbacks.set(callbackId, internalCallback);
        this.notificationCenter.on(eventName, internalCallback);
        return this.listenerCounter++;
    }
    off(eventName, listenerId) {
        if (listenerId === undefined) {
            this.notificationCenter.off(eventName);
            this.callbacks.deleteCallbackMap(eventName);
            return;
        }
        const callbackId = [eventName, listenerId];
        const internalCallback = this.callbacks.get(callbackId);
        if (!internalCallback) {
            return;
        }
        this.notificationCenter.off(eventName, internalCallback);
        this.callbacks.delete(callbackId);
    }
    emit(dispatchableEvent) {
        const internalEventData = {
            eventName: dispatchableEvent.name,
            object: this.notificationCenter,
            data: Object.assign({}, dispatchableEvent),
        };
        try {
            this.notificationCenter.notify(internalEventData);
        }
        catch (err) {
            if (err instanceof TypeError) {
                return;
            }
            throw err;
        }
    }
    hasListeners(eventName) {
        return this.notificationCenter.hasListeners(eventName);
    }
}
class CallbackStore {
    constructor() {
        this.callbackTree = {};
    }
    set(callbackId, internalCallback) {
        const [eventName, listenerId] = callbackId;
        if (!this.callbackTree[eventName]) {
            this.callbackTree[eventName] = new Map();
        }
        this.callbackTree[eventName].set(listenerId, internalCallback);
    }
    get(callbackId) {
        const [eventName, listenerId] = callbackId;
        const callbackMap = this.callbackTree[eventName];
        if (!callbackMap) {
            return null;
        }
        const internalCallback = callbackMap.get(listenerId);
        return internalCallback ? internalCallback : null;
    }
    delete(callbackId) {
        const [eventName, listenerId] = callbackId;
        const callbackMap = this.callbackTree[eventName];
        if (!callbackMap) {
            return;
        }
        callbackMap.delete(listenerId);
        if (callbackMap.size === 0) {
            delete this.callbackTree[eventName];
        }
    }
    deleteCallbackMap(eventName) {
        const callbackMap = this.callbackTree[eventName];
        if (!callbackMap) {
            return;
        }
        callbackMap.clear();
        delete this.callbackTree[eventName];
    }
}
//# sourceMappingURL=internal-event-manager.js.map