import { plannedTasksDB, } from "../persistence/planned-tasks-store";
import { taskScheduler as getTaskScheduler, } from "./schedulers/time-based";
import { off, on } from "../events";
import { PlanningType } from "./planner/planned-task";
import { getLogger } from "../utils/logger";
import { TaskChain, TaskResultStatus } from "./task-chain";
export class TaskCancelManager {
    constructor(taskStore = plannedTasksDB, taskScheduler) {
        this.taskStore = taskStore;
        this.taskScheduler = taskScheduler;
        this.cancelEvents = new Set();
        this.logger = getLogger("TaskCancelManager");
    }
    async init() {
        const cancelEvents = await this.taskStore.getAllCancelEvents();
        cancelEvents.forEach((cancelEvent) => {
            this.listenToCancelEvent(cancelEvent);
        });
    }
    add(plannedTask) {
        this.listenToCancelEvent(plannedTask.cancelEvent);
    }
    listenToCancelEvent(cancelEvent) {
        if (this.cancelEvents.has(cancelEvent)) {
            return;
        }
        this.cancelEvents.add(cancelEvent);
        const listenerId = on(cancelEvent, (evt) => {
            const evtName = evt.name;
            off(evtName, listenerId);
            this.cancelEvents.delete(evtName);
            this.cancelByEventName(evtName)
                .then(() => {
                TaskChain.finalize(evt.id, TaskResultStatus.Ok);
            })
                .catch((err) => {
                TaskChain.finalize(evt.id, TaskResultStatus.Error, err);
            });
        });
    }
    async cancelByEventName(eventName) {
        const tasks = await this.taskStore.getAllFilteredByCancelEvent(eventName);
        for (const task of tasks) {
            await this.cancelTask(task);
        }
    }
    async cancelTask(task) {
        try {
            if (task.planningType === PlanningType.Scheduled) {
                await this.getTaskScheduler().cancel(task.id);
            }
            else {
                await this.taskStore.delete(task.id);
            }
        }
        catch (err) {
            const { id, name, startAt, interval, recurrent } = task;
            this.logger.error(`Error canceling task: ${JSON.stringify({
                id,
                name,
                startAt,
                interval,
                recurrent,
            })} -> ${err}`);
        }
    }
    getTaskScheduler() {
        if (!this.taskScheduler) {
            this.taskScheduler = getTaskScheduler();
        }
        return this.taskScheduler;
    }
}
export const taskCancelManager = new TaskCancelManager();
//# sourceMappingURL=cancel-manager.js.map