import { getTask } from "../provider";
export class TaskGraphBrowser {
    constructor(taskProvider = getTask) {
        this.taskProvider = taskProvider;
        this.entries = new Map();
    }
    addEntry(invocationEvent, runnableTask) {
        if (!this.entries.has(invocationEvent)) {
            this.entries.set(invocationEvent, []);
        }
        this.entries.get(invocationEvent).push(runnableTask);
    }
    getTriggeredBy(invocationEvent) {
        if (!this.entries.has(invocationEvent)) {
            return [];
        }
        return this.entries.get(invocationEvent);
    }
    getUniques() {
        const uniques = new Set();
        for (let runnableTasks of this.entries.values()) {
            for (let runnableTask of runnableTasks) {
                uniques.add(runnableTask);
            }
        }
        return [...uniques];
    }
    any(matcher) {
        for (let runnableTask of this.getUniques()) {
            const instance = this.taskProvider(runnableTask.name);
            const matches = matcher(Object.assign(Object.assign({}, runnableTask), { instance }));
            if (matches) {
                return true;
            }
        }
        return false;
    }
    anyFrom(eventName, matcher) {
        const memory = new Map();
        try {
            return this.memoizedAnyFrom(eventName, matcher, memory);
        }
        catch (err) {
            if (err instanceof RangeError) {
                throw new Error("Task graphs with cycles are not supported");
            }
            else {
                throw err;
            }
        }
    }
    depict() {
        const rootEvents = this.getRootEvents();
        return rootEvents.map((eventName) => ({
            trigger: eventName,
            tasks: this.walkFrom(eventName),
        }));
    }
    walkFrom(eventName) {
        const memory = new Map();
        try {
            return this.memoizedWalk(eventName, memory);
        }
        catch (err) {
            if (err instanceof RangeError) {
                throw new Error("Task graphs with cycles are not supported");
            }
            else {
                throw err;
            }
        }
    }
    getRootEvents() {
        const rootEvents = new Set(this.entries.keys());
        for (let runnableTasks of this.entries.values()) {
            for (let runnableTask of runnableTasks) {
                const task = this.taskProvider(runnableTask.name);
                for (let outputEvent of task.outputEventNames) {
                    if (rootEvents.has(outputEvent)) {
                        rootEvents.delete(outputEvent);
                    }
                }
            }
        }
        return [...rootEvents];
    }
    memoizedAnyFrom(eventName, matcher, memory) {
        if (!this.entries.has(eventName)) {
            return false;
        }
        if (memory.has(eventName)) {
            return memory.get(eventName);
        }
        const runnableTasks = this.entries.get(eventName);
        for (let runnableTask of runnableTasks) {
            const instance = this.taskProvider(runnableTask.name);
            const matches = matcher(Object.assign(Object.assign({}, runnableTask), { instance }));
            if (matches) {
                return true;
            }
            for (let outputEvent of instance.outputEventNames) {
                const childMatches = this.memoizedAnyFrom(outputEvent, matcher, memory);
                if (childMatches) {
                    return true;
                }
            }
        }
        return false;
    }
    memoizedWalk(eventName, memory) {
        if (!this.entries.has(eventName)) {
            return [];
        }
        if (memory.has(eventName)) {
            return memory.get(eventName);
        }
        const runnableTasks = this.entries.get(eventName);
        const eventTriggeredTasks = runnableTasks.map((runnableTask) => (Object.assign(Object.assign({}, runnableTask), { outputs: this.walkOutputs(runnableTask, memory) })));
        memory.set(eventName, eventTriggeredTasks);
        return eventTriggeredTasks;
    }
    walkOutputs(runnableTask, memory) {
        const task = this.taskProvider(runnableTask.name);
        return task.outputEventNames.map((outputEvent) => ({
            trigger: outputEvent,
            tasks: this.memoizedWalk(outputEvent, memory),
        }));
    }
}
export const taskGraphBrowser = new TaskGraphBrowser();
//# sourceMappingURL=browser.js.map