import { getLogger } from "../../utils/logger";
import { getTask, checkIfTaskExists } from "../provider";
import { taskCancelManager } from "../cancel-manager";
import { taskGraphBrowser } from "./browser";
import { on } from "../../events";
import { run } from "..";
export class TaskGraphLoader {
    constructor(taskEventBinder = on, runnableTaskDescriptor = run, taskVerifier = checkIfTaskExists, taskProvider = getTask, cancelManager = taskCancelManager, graphBrowser = taskGraphBrowser) {
        this.taskEventBinder = taskEventBinder;
        this.runnableTaskDescriptor = runnableTaskDescriptor;
        this.taskVerifier = taskVerifier;
        this.taskProvider = taskProvider;
        this.cancelManager = cancelManager;
        this.graphBrowser = graphBrowser;
        this.graphTasks = new Set();
    }
    async load(graph) {
        if (this.loadingTaskGraph) {
            throw new Error("Loading more than one task graph is not permitted");
        }
        const createEventListener = (eventName, taskBuilder) => this.bindTaskToStartAndCancelEvent(eventName, taskBuilder);
        const planTaskToBeRun = (taskName, taskParams) => this.trackTaskGoingToBeRun(taskName, taskParams);
        this.getLogger().info("Loading task graph");
        this.loadingTaskGraph = graph.describe(createEventListener, planTaskToBeRun);
        await this.loadingTaskGraph;
        await this.cancelManager.init();
    }
    async isReady() {
        const tasksToBePrepared = await this.tasksNotReady();
        return tasksToBePrepared.length === 0;
    }
    async prepare() {
        const tasksToBePrepared = await this.tasksNotReady();
        this.getLogger().info(`${tasksToBePrepared.length} are up to be prepared`);
        for (let task of tasksToBePrepared) {
            const hasYetToBePrepared = await this.hasToBePrepared(task);
            if (hasYetToBePrepared) {
                await task.prepare();
            }
        }
    }
    async tasksNotReady() {
        if (!this.loadingTaskGraph) {
            throw new Error("Load a task graph first!");
        }
        await this.loadingTaskGraph;
        const tasksToBePrepared = [];
        for (const task of this.graphTasks) {
            const hasToBePrepared = await this.hasToBePrepared(task);
            if (hasToBePrepared) {
                tasksToBePrepared.push(task);
            }
        }
        return tasksToBePrepared;
    }
    bindTaskToStartAndCancelEvent(eventName, taskBuilder) {
        this.graphBrowser.addEntry(eventName, taskBuilder.build());
        this.taskEventBinder(eventName, taskBuilder);
    }
    trackTaskGoingToBeRun(taskName, taskParams) {
        this.taskVerifier(taskName);
        this.graphTasks.add(this.taskProvider(taskName));
        return this.runnableTaskDescriptor(taskName, taskParams);
    }
    async hasToBePrepared(task) {
        try {
            await task.checkIfCanRun();
            return false;
        }
        catch (err) {
            return true;
        }
    }
    getLogger() {
        if (!this.logger) {
            this.logger = getLogger("TaskGraphLoader");
        }
        return this.logger;
    }
}
export const taskGraph = new TaskGraphLoader();
//# sourceMappingURL=loader.js.map