import { taskScheduler as getTaskScheduler, } from "../schedulers/time-based";
import { InstantTaskRunner, } from "../schedulers/immediate/instant-task-runner";
import { plannedTasksDB, } from "../../persistence/planned-tasks-store";
import { taskCancelManager } from "../cancel-manager";
import { taskGraphBrowser } from "../graph/browser";
import { checkIfTaskExists } from "../provider";
import { TaskChain, TaskResultStatus } from "../task-chain";
import { TaskPlannerParallelizer } from "./parallelizer";
export class TaskPlanner {
    constructor(taskScheduler, taskRunner = new InstantTaskRunner(plannedTasksDB), taskStore = plannedTasksDB, cancelManager = taskCancelManager, parallelizer = new TaskPlannerParallelizer(taskGraphBrowser)) {
        this.taskScheduler = taskScheduler;
        this.taskRunner = taskRunner;
        this.taskStore = taskStore;
        this.cancelManager = cancelManager;
        this.parallelizer = parallelizer;
    }
    async plan(runnableTask, dispatchableEvent) {
        const invocationEvent = this.parallelizer.spawnChildEvent(dispatchableEvent);
        try {
            checkIfTaskExists(runnableTask.name);
            return await (runnableTask.interval > 0 || runnableTask.startAt !== -1
                ? this.planScheduled(runnableTask, invocationEvent)
                : this.planImmediate(runnableTask, invocationEvent));
        }
        catch (err) {
            emitTaskChainFinished(invocationEvent, err);
            throw err;
        }
    }
    async planImmediate(runnableTask, dispatchableEvent) {
        const existedBefore = await this.taskStore.get(runnableTask);
        const plannedTask = await this.taskRunner.run(runnableTask, dispatchableEvent);
        if (!existedBefore) {
            this.cancelManager.add(plannedTask);
        }
        return plannedTask;
    }
    async planScheduled(runnableTask, dispatchableEvent) {
        const possibleExisting = await this.taskStore.get(runnableTask);
        if (possibleExisting) {
            emitTaskChainFinished(dispatchableEvent);
            return possibleExisting;
        }
        const plannedTask = await this.getTaskScheduler().schedule(runnableTask);
        emitTaskChainFinished(dispatchableEvent);
        this.cancelManager.add(plannedTask);
        return plannedTask;
    }
    getTaskScheduler() {
        if (!this.taskScheduler) {
            this.taskScheduler = getTaskScheduler();
        }
        return this.taskScheduler;
    }
}
function emitTaskChainFinished(dispatchableEvent, error) {
    if (!dispatchableEvent) {
        return;
    }
    const status = error ? TaskResultStatus.Error : TaskResultStatus.Ok;
    TaskChain.finalize(dispatchableEvent.id, status, error);
}
//# sourceMappingURL=index.js.map