import { getTask } from "../provider";
import { on, TaskDispatcherEvent, off } from "../../events";
import { getLogger } from "../../utils/logger";
import { now } from "../../utils/time";
import { lastRecurrentRun } from "./last-recurrent-run";
const FAILURE_THRESHOLD = 3;
export class SingleTaskRunner {
    constructor(taskStore) {
        this.taskStore = taskStore;
        this.logger = getLogger("SingleTaskRunner");
    }
    async run(plannedTask, startEvent) {
        const { name, id, params } = plannedTask;
        const task = getTask(name);
        const lastRunOffset = plannedTask.recurrent
            ? lastRecurrentRun.timeSince()
            : 0;
        await this.taskStore.updateLastRun(id, now() - lastRunOffset);
        try {
            const parameterizedTask = new ParameterizedTask(task, params, startEvent);
            await this.runWithTimeout(id, parameterizedTask, startEvent.id);
        }
        catch (error) {
            await this.taskStore.increaseErrorCount(id);
        }
        if (!plannedTask.recurrent && plannedTask.interval > 0) {
            await this.handleOneShotTask(plannedTask.id);
        }
    }
    runWithTimeout(id, task, startEventId) {
        return new Promise((resolve, reject) => {
            const listenerId = on(TaskDispatcherEvent.TaskExecutionTimedOut, (evt) => {
                if (evt.id === startEventId) {
                    off(TaskDispatcherEvent.TaskExecutionTimedOut, listenerId);
                    task.cancel();
                    this.taskStore.increaseTimeoutCount(id).then(() => resolve());
                }
            });
            let taskAlreadyRun = false;
            let taskChainFinished = false;
            this.waitForTaskChainToFinish(startEventId).then(() => {
                taskChainFinished = true;
                if (taskAlreadyRun) {
                    resolve();
                }
            });
            task
                .run()
                .then(() => {
                taskAlreadyRun = true;
                off(TaskDispatcherEvent.TaskExecutionTimedOut, listenerId);
                if (taskChainFinished) {
                    resolve();
                }
            })
                .catch((err) => {
                off(TaskDispatcherEvent.TaskExecutionTimedOut, listenerId);
                reject(err);
            });
        });
    }
    waitForTaskChainToFinish(startEventId) {
        return new Promise((resolve) => {
            const listenerId = on(TaskDispatcherEvent.TaskChainFinished, (chainFinishedEvt) => {
                if (chainFinishedEvt.id === startEventId) {
                    off(TaskDispatcherEvent.TaskChainFinished, listenerId);
                    resolve();
                }
            });
        });
    }
    async handleOneShotTask(id) {
        const { errorCount, timeoutCount } = await this.taskStore.get(id);
        if (errorCount === 0 && timeoutCount === 0) {
            await this.taskStore.delete(id);
        }
        else if (errorCount > FAILURE_THRESHOLD ||
            timeoutCount > FAILURE_THRESHOLD) {
            this.logger.warn(`One-shot planned task id:${id} discarded after {errorCount:${errorCount}, timeoutCount:${timeoutCount}}`);
            await this.taskStore.delete(id);
        }
    }
}
class ParameterizedTask {
    constructor(task, taskParams, startEvent) {
        this.task = task;
        this.taskParams = taskParams;
        this.startEvent = startEvent;
    }
    run() {
        return this.task.run(this.taskParams, this.startEvent);
    }
    cancel() {
        this.task.cancel();
    }
}
//# sourceMappingURL=single-task-runner.js.map