import { getLogger } from "../../../../utils/logger";
import { unpackTaskChainRunnerServiceIntent, } from "./intents.android";
import { createEvent, TaskDispatcherEvent, emit, on, off, } from "../../../../events";
import { now } from "../../../../utils/time";
const TIMEOUT = 180000;
const TIMEOUT_EVENT_OFFSET = 5000;
const IDLE_TIME = 1000;
export class TaskChainRunnerService {
    onCreate(nativeService) {
        this.nativeService = nativeService;
        this.wakeLock = taskChainRunnerWakeLock(nativeService);
        this.wakeLock.setReferenceCounted(false);
        this.wakeLock.acquire();
        this.timeoutIds = new Set();
        this.taskChainCount = 0;
        this.killed = false;
        this.logger = getLogger("TaskChainRunnerService");
        this.logger.debug("onCreate called");
        this.initializeExecutionWindow();
    }
    onStartCommand(intent, flags, startId) {
        this.logger.info(`Service called {flags=${flags}, startId=${startId}}`);
        if (intent) {
            this.taskChainStarted();
            this.processRequest(intent)
                .then(() => {
                this.logger.debug("Task chain finished running");
                this.taskChainDone();
            })
                .catch((err) => {
                this.logger.error(`Error while executing the task chain: ${err}`);
                this.taskChainDone();
            });
        }
        return android.app.Service.START_REDELIVER_INTENT;
    }
    onDestroy() {
        this.logger.debug("onDestroy called");
    }
    initializeExecutionWindow() {
        this.executionStart = now();
        this.wakeLock.acquire(TIMEOUT);
    }
    async processRequest(intent) {
        const params = this.extractRequestParams(intent);
        const [launchEvent, timeoutId] = this.prepareTaskChainExecution(params);
        await this.startTaskChainExecution(launchEvent, timeoutId);
        await this.stayIdleIfPossible(IDLE_TIME);
    }
    taskChainStarted() {
        this.taskChainCount++;
    }
    taskChainDone() {
        this.taskChainCount--;
        if (this.taskChainCount === 0) {
            this.gracefullyStop();
        }
    }
    extractRequestParams(intent) {
        const params = unpackTaskChainRunnerServiceIntent(intent);
        this.logger.info(`Request params {launchEvent=${params.launchEvent}, eventData=${JSON.stringify(params.eventData)}${params.eventId ? `, eventId=${params.eventId}` : ""}}`);
        return params;
    }
    prepareTaskChainExecution(params) {
        const timeout = this.calculateTimeout();
        const launchEvent = createEvent(params.launchEvent, {
            data: params.eventData,
            expirationTimestamp: this.getExpirationTimestamp(timeout),
        });
        if (params.eventId) {
            launchEvent.id = params.eventId;
        }
        const { id } = launchEvent;
        const timeoutEvent = createEvent(TaskDispatcherEvent.TaskExecutionTimedOut, {
            id,
        });
        this.logger.info(`Execution will timeout in ${timeout}, for tasks running with execution id: ${id}`);
        const timeoutId = setTimeout(() => {
            emit(timeoutEvent);
            this.logger.warn(`Execution timed out for tasks running with execution id: ${id}`);
        }, timeout);
        this.timeoutIds.add(timeoutId);
        return [launchEvent, timeoutId];
    }
    startTaskChainExecution(launchEvent, timeoutId) {
        return new Promise((resolve) => {
            const listenerId = on(TaskDispatcherEvent.TaskChainFinished, (evt) => {
                if (evt.id === launchEvent.id) {
                    off(TaskDispatcherEvent.TaskChainFinished, listenerId);
                    this.logger.info(`Task chain with id (${launchEvent.id}) finished its execution`);
                    this.timeoutIds.delete(timeoutId);
                    clearTimeout(timeoutId);
                    resolve();
                }
            });
            emit(launchEvent);
        });
    }
    stayIdleIfPossible(idleTime) {
        const timeout = this.calculateTimeout();
        if (timeout < idleTime) {
            return Promise.resolve();
        }
        return new Promise((resolve) => setTimeout(resolve, idleTime));
    }
    calculateTimeout() {
        const currentMillis = now();
        const diff = currentMillis - this.executionStart;
        return TIMEOUT - TIMEOUT_EVENT_OFFSET - diff;
    }
    getExpirationTimestamp(timeout) {
        return now() + timeout;
    }
    gracefullyStop() {
        this.logger.debug("Stopping service");
        this.killWithFire();
        for (const timeoutId of this.timeoutIds) {
            clearTimeout(timeoutId);
        }
        if (this.wakeLock.isHeld()) {
            this.wakeLock.release();
            this.logger.debug("Lock released");
        }
    }
    killWithFire() {
        let startId = 1;
        while (!this.killed) {
            const last = this.nativeService.stopSelfResult(startId);
            if (last) {
                this.killed = true;
            }
            else {
                startId++;
            }
        }
        this.logger.info(`Done running (startId=${startId})`);
    }
}
function taskChainRunnerWakeLock(context) {
    const wakeLockName = "TaskDispatcher::TaskChainRunnerWakeLock";
    const powerManager = context.getSystemService(android.content.Context.POWER_SERVICE);
    return powerManager.newWakeLock(android.os.PowerManager.PARTIAL_WAKE_LOCK, wakeLockName);
}
let _taskChainRunnerService;
export function getTaskChainRunnerService() {
    if (!_taskChainRunnerService) {
        _taskChainRunnerService = new TaskChainRunnerService();
    }
    return _taskChainRunnerService;
}
//# sourceMappingURL=runner-service.android.js.map