import { Application } from "@nativescript/core";
import { unpackAlarmRunnerServiceIntent } from "../../intents.android";
import { AndroidNotification, createNotification, setupNotificationChannels, } from "../../notification-manager.android";
import { plannedTasksDB, } from "../../../../../../persistence/planned-tasks-store";
import { BatchTaskRunner } from "../../../../../runners/batch-task-runner";
import { TaskDispatcherEvent, emit, createEvent, } from "../../../../../../events";
import { TaskManager } from "../../../../../manager";
import { PlanningType } from "../../../../../planner/planned-task";
import { getLogger } from "../../../../../../utils/logger";
import { now } from "../../../../../../utils/time";
import { ForegroundChecker } from "../../../../../foreground-checker";
const MIN_TIMEOUT = 60000;
const TIMEOUT_EVENT_OFFSET = 5000;
export class AlarmRunnerService {
    onCreate(nativeService) {
        this.nativeService = nativeService;
        this.runsInForeground = false;
        this.timeOffset = 0;
        this.currentTime = now();
        this.started = false;
        this.inForeground = false;
        this.wakeLock = alarmRunnerWakeLock(nativeService);
        this.wakeLock.acquire();
        this.wakeLock.setReferenceCounted(false);
        this.taskStore = plannedTasksDB;
        this.foregroundChecker = new ForegroundChecker();
        this.logger = getLogger("AlarmRunnerService");
        this.logger.debug("onCreate called");
    }
    onStartCommand(intent, flags, startId) {
        this.logger.info(`Service called {flags=${flags}, startId=${startId}}`);
        const startFlag = android.app.Service.START_REDELIVER_INTENT;
        if (this.alreadyRunning(startId)) {
            return startFlag;
        }
        this.extractInvocationArguments(intent);
        if (this.runsInForeground) {
            this.runInForeground();
        }
        this.runTasks()
            .then(() => {
            this.logger.debug("Tasks finished running");
            this.gracefullyStop();
        })
            .catch((err) => {
            this.logger.error(`Error while running tasks ${err}`);
            this.gracefullyStop();
        });
        return startFlag;
    }
    onDestroy() {
        this.logger.debug("onDestroy called");
        this.gracefullyStop();
        this.nativeService = null;
    }
    alreadyRunning(startId) {
        if (startId === 1) {
            this.started = true;
            this.logger.debug("Service started");
            return false;
        }
        this.logger.warn(`Service already running! Dismissing call -> startId: ${startId}!`);
        this.nativeService.stopSelf(startId);
        return true;
    }
    extractInvocationArguments(intent) {
        const args = unpackAlarmRunnerServiceIntent(intent);
        this.logger.info(`InvocationParams {runsInForeground=${args.runInForeground}, offset=${args.timeOffset}, invocationTime=${args.invocationTime}}`);
        this.runsInForeground = args.runInForeground;
        this.timeOffset = args.timeOffset;
        this.currentTime = args.invocationTime;
    }
    runInForeground() {
        if (this.inForeground) {
            return;
        }
        this.nativeService.startForeground(AndroidNotification.LocationUsage, createNotification(this.nativeService, AndroidNotification.LocationUsage));
        this.inForeground = true;
        this.logger.debug("Running in foreground");
    }
    async runTasks() {
        const taskManager = this.createTaskManager();
        const tasksToRun = await taskManager.tasksToRun();
        const taskCount = tasksToRun.length;
        if (taskCount > 0) {
            const timeout = await this.calculateTimeout(taskManager);
            const executionStartedEvt = this.initializeExecutionWindow(timeout);
            const taskRunner = new BatchTaskRunner(this.taskStore);
            this.logger.info(`Running ${taskCount} tasks`);
            await taskRunner.run(tasksToRun, executionStartedEvt);
        }
        else {
            this.logger.warn("Service was called but no tasks were run!");
        }
    }
    createTaskManager() {
        return new TaskManager(PlanningType.Scheduled, this.taskStore, this.foregroundChecker, this.timeOffset, this.currentTime);
    }
    initializeExecutionWindow(timeout) {
        this.wakeLock.acquire(timeout);
        const executionTimeout = timeout - TIMEOUT_EVENT_OFFSET;
        const startEvent = createEvent(TaskDispatcherEvent.TaskExecutionStarted, {
            expirationTimestamp: this.getExpirationTimestamp(executionTimeout),
        });
        const { id } = startEvent;
        const timeoutEvent = createEvent(TaskDispatcherEvent.TaskExecutionTimedOut, {
            id,
        });
        this.logger.info(`Execution will timeout in ${executionTimeout}, for tasks running with execution id: ${id}`);
        this.timeoutId = setTimeout(() => {
            emit(timeoutEvent);
            this.logger.warn(`Execution timed out for tasks running with execution id: ${id}`);
        }, executionTimeout);
        return startEvent;
    }
    async calculateTimeout(taskPlanner) {
        const nextExecutionTime = await taskPlanner.nextInterval();
        return Math.max(nextExecutionTime, MIN_TIMEOUT);
    }
    getExpirationTimestamp(timeout) {
        return now() + timeout;
    }
    gracefullyStop() {
        this.moveToBackground();
        if (this.started) {
            this.logger.debug("Stopping service");
            this.killWithFire();
            this.started = false;
        }
        if (this.timeoutId) {
            clearTimeout(this.timeoutId);
        }
        if (this.wakeLock.isHeld()) {
            this.wakeLock.release();
            this.logger.debug("Lock released");
        }
    }
    moveToBackground() {
        if (!this.inForeground) {
            return;
        }
        const andRemoveNotification = true;
        this.nativeService.stopForeground(andRemoveNotification);
        this.inForeground = false;
        this.logger.debug("Running in background");
    }
    killWithFire() {
        let startId = 1;
        let last = false;
        while (!last) {
            last = this.nativeService.stopSelfResult(startId);
            if (!last) {
                startId++;
            }
        }
        this.logger.info(`Done running (startId=${startId})`);
    }
}
function alarmRunnerWakeLock(context) {
    const wakeLockName = "TaskDispatcher::AlarmRunnerWakeLock";
    const powerManager = context.getSystemService(android.content.Context.POWER_SERVICE);
    return powerManager.newWakeLock(android.os.PowerManager.PARTIAL_WAKE_LOCK, wakeLockName);
}
let _alarmRunnerService;
export function getAlarmRunnerService() {
    if (!_alarmRunnerService) {
        setupNotificationChannels(Application.android.context);
        _alarmRunnerService = new AlarmRunnerService();
    }
    return _alarmRunnerService;
}
//# sourceMappingURL=runner-service.android.js.map