import { Utils } from "@nativescript/core";
import { createAppLaunchIntent } from "./intents.android";
import { getLogger } from "../../../../utils/logger";
export var AndroidNotification;
(function (AndroidNotification) {
    AndroidNotification[AndroidNotification["LocationUsage"] = 1000] = "LocationUsage";
})(AndroidNotification || (AndroidNotification = {}));
let _notificationChannels;
function notificationChannels(context) {
    if (!_notificationChannels) {
        const getString = createStringFetcher(context);
        const NotificationManagerCompat = androidx.core.app.NotificationManagerCompat;
        _notificationChannels = new Map();
        try {
            _notificationChannels.set(AndroidNotification.LocationUsage, {
                id: "LOCATION_USAGE",
                name: getString("task_dispatcher_channel_name"),
                description: getString("task_dispatcher_channel_description"),
                priority: NotificationManagerCompat.IMPORTANCE_LOW,
            });
        }
        catch (e) {
            logger().debug(`No strings set for ${AndroidNotification.LocationUsage} channel. Skipping...`);
        }
    }
    return _notificationChannels;
}
export function setupNotificationChannels(context) {
    const sdkInt = android.os.Build.VERSION.SDK_INT;
    if (sdkInt < 26) {
        return;
    }
    for (const channel of notificationChannels(context).values()) {
        setupNotificationChannel(context, channel);
    }
}
export function createNotification(context, type) {
    if (!notificationChannels(context).has(type)) {
        throw new Error(`Translation strings had not been setup for ${AndroidNotification[type]} notification type. Please follow setup instructions.`);
    }
    let notificationBuilder;
    switch (type) {
        case AndroidNotification.LocationUsage:
            const appLaunchIntent = createAppLaunchIntent(context);
            const pendingIntent = android.app.PendingIntent.getActivity(context, 0, appLaunchIntent, 0);
            notificationBuilder = initializeNotificationBuilder(context, type, "task_dispatcher_notification_title", "task_dispatcher_notification_content").setContentIntent(pendingIntent);
            break;
    }
    return notificationBuilder ? notificationBuilder.build() : null;
}
function setupNotificationChannel(context, channel) {
    const notificationManager = context.getSystemService(android.app.NotificationManager.class);
    if (!notificationManager) {
        return;
    }
    const { id, name, description, priority } = channel;
    const ch = new android.app.NotificationChannel(id, name, priority);
    ch.setDescription(description);
    notificationManager.createNotificationChannel(ch);
}
function initializeNotificationBuilder(context, type, title, content) {
    const getIcon = createIconFetcher(context);
    const { id, priority } = notificationChannels(context).get(type);
    const iconId = getIcon();
    const getString = createStringFetcher(context);
    return new androidx.core.app.NotificationCompat.Builder(context, id)
        .setSmallIcon(iconId)
        .setContentTitle(getString(title))
        .setContentText(getString(content))
        .setPriority(priority);
}
function createStringFetcher(context) {
    return (key) => context.getResources().getString(Utils.android.resources.getStringId(key));
}
function createIconFetcher(context) {
    const getDrawableId = createDrawableIdFetcher(context);
    return (key) => {
        let icon = 0;
        if (key && key.indexOf(Utils.RESOURCE_PREFIX) === 0) {
            icon = getDrawableId(key.substr(Utils.RESOURCE_PREFIX.length));
        }
        if (icon === 0 && android.os.Build.VERSION.SDK_INT >= 21) {
            icon = getDrawableId("ic_stat_notify_silhouette");
        }
        if (icon === 0) {
            icon = getDrawableId("ic_stat_notify");
        }
        if (icon === 0) {
            icon = context.getApplicationInfo().icon;
        }
        return icon;
    };
}
function createDrawableIdFetcher(context) {
    const resources = context.getResources();
    const packageName = context.getApplicationInfo().packageName;
    return (key) => {
        return resources.getIdentifier(key, "drawable", packageName);
    };
}
let _logger;
function logger() {
    if (!_logger) {
        _logger = getLogger("NotificationManager");
    }
    return _logger;
}
//# sourceMappingURL=notification-manager.android.js.map