import { emit, hasListeners } from "../events";
import { getLogger } from "../utils/logger";
import { TaskChain, TaskResultStatus } from "./task-chain";
import { now } from "../utils/time";
import { flatten } from "../utils/serialization";
export class Task {
    constructor(name, taskConfig = {}) {
        this._name = name;
        this._executionHistory = new Set();
        this._cancelFunctions = new Map();
        this.configureTask(taskConfig);
    }
    get name() {
        return this._name;
    }
    get outputEventNames() {
        return this._taskConfig.outputEventNames;
    }
    async run(taskParams, invocationEvent) {
        if (this._invocationEvent && !this.isDone()) {
            this.cancelParallelInvocation(taskParams, invocationEvent);
            return;
        }
        this._taskParams = taskParams;
        this._invocationEvent = invocationEvent;
        this.log(`Run triggered by ${invocationEvent.name} event ${invocationEvent.expirationTimestamp !== -1
            ? `with ${invocationEvent.expirationTimestamp} expirationTimestamp`
            : ``}`);
        try {
            await this.checkIfCanRun();
            const outcome = await this.onRun(taskParams, invocationEvent);
            if (!this.isDone()) {
                this.processTaskOutcome(outcome);
            }
        }
        catch (err) {
            this.getLogger().error(`Execution failed with params ${JSON.stringify(taskParams)} and invocation event ${JSON.stringify(invocationEvent)}: ${err.stack ? err.stack : err}`);
            this.emitEndEvent(TaskResultStatus.Error, err);
            throw err;
        }
        this.removeCancelFunction();
    }
    runsInBackground() {
        return !this._taskConfig.foreground;
    }
    cancel() {
        this.emitEndEvent(TaskResultStatus.Cancelled);
        if (this._cancelFunctions.has(this._invocationEvent.id)) {
            const cancelFunction = this._cancelFunctions.get(this._invocationEvent.id);
            cancelFunction();
            this.removeCancelFunction();
        }
        this.log("Cancelled");
    }
    checkIfCanRun() {
        return Promise.resolve();
    }
    prepare() {
        return Promise.resolve();
    }
    runAgainIn(seconds, params) {
        defer(this.name, seconds, params ? params : this._taskParams);
        this.log(`Will run again in ${seconds} s`);
    }
    setCancelFunction(f) {
        if (this.isDone()) {
            f();
            return;
        }
        this._cancelFunctions.set(this._invocationEvent.id, f);
    }
    log(message) {
        this.getLogger().info(`${message} (invocationId=${this._invocationEvent.id})`);
    }
    remainingTime() {
        if (this._invocationEvent.expirationTimestamp === -1) {
            return -1;
        }
        let timeForExpiration = this._invocationEvent.expirationTimestamp - now();
        if (this.outputEventNames.some(hasListeners)) {
            timeForExpiration *= 0.9;
        }
        return Math.floor(timeForExpiration);
    }
    configureTask(taskConfig) {
        this._taskConfig = {
            foreground: taskConfig.foreground,
            outputEventNames: taskConfig.outputEventNames && taskConfig.outputEventNames.length > 0
                ? taskConfig.outputEventNames
                : [`${this.name}Finished`],
        };
    }
    cancelParallelInvocation(taskParams, invocationEvent) {
        this.getLogger().warn(`Multiple executions of a task cannot run concurrently. Warning triggered with: params=${JSON.stringify(taskParams)} and invocationEvent=${JSON.stringify(invocationEvent)}`);
        const reason = new Error("Concurrent Execution: Executing multiple instances of a task concurrently is not allowed");
        this.emitEndEvent(TaskResultStatus.Cancelled, reason, invocationEvent.id);
    }
    processTaskOutcome(outcome) {
        if (this.outputEventNames.length > 1 && (!outcome || !outcome.eventName)) {
            this.getLogger().error(`Task was declared to have more than one possible output event, but outcome did not specify any.
        Please, check if you forgot to fill eventName field in any of the outputs of your conditional branches.
        Received outcome: ${JSON.stringify(outcome)}`);
            const reason = new Error("Cannot choose which event to emit");
            this.emitEndEvent(TaskResultStatus.Error, reason);
            return;
        }
        const outputEvent = !outcome || !outcome.eventName
            ? this.outputEventNames[0]
            : outcome.eventName;
        if (!outcome || !outcome.result) {
            this.done(outputEvent);
        }
        else {
            this.done(outputEvent, outcome.result);
        }
    }
    done(eventName, result) {
        if (this.isDone()) {
            return;
        }
        this.markAsDone();
        if (!this.outputEventNames.find((name) => name === eventName)) {
            this.getLogger().warn(`About to emit an event (${eventName}) not declared in the list of possible output events of this task.
      This can lead to unintended behaviors. Perhaps you have forgot to include it in the task configuration or there is a typo`);
        }
        if (!hasListeners(eventName)) {
            this.emitEndEvent(TaskResultStatus.Ok);
            return;
        }
        const data = result
            ? typeof result === "object"
                ? flatten(result)
                : { result }
            : {};
        emit({
            name: eventName,
            id: this._invocationEvent.id,
            expirationTimestamp: this._invocationEvent.expirationTimestamp,
            data,
        });
        this.log(`Finished running with ${eventName} event`);
    }
    isDone() {
        return this._executionHistory.has(this._invocationEvent.id);
    }
    markAsDone(invocationId) {
        const id = invocationId ? invocationId : this._invocationEvent.id;
        this._executionHistory.add(id);
    }
    emitEndEvent(status, err, invocationId) {
        const id = invocationId ? invocationId : this._invocationEvent.id;
        this.markAsDone(id);
        TaskChain.finalize(id, status, err);
    }
    removeCancelFunction() {
        this._cancelFunctions.delete(this._invocationEvent.id);
    }
    getLogger() {
        if (!this._logger) {
            this._logger = getLogger(`Task (${this._name})`);
        }
        return this._logger;
    }
}
let defer;
export function setTaskDeferrer(taskDeferrer) {
    defer = taskDeferrer;
}
//# sourceMappingURL=task.js.map